<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_stub_class_id" />

<xsl:template match="/">
  <xsl:for-each select="/nerv_icde/component/stub_list/row[id=$export_stub_class_id]">
    <xsl:variable name="method_domain_id" select="method_domain_id" />
    <xsl:variable name="interface_name" select="interface_name" />
    <xsl:variable name="class_name" select="class_name" />

    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="$method_domain_id" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>

#ifndef Define__Nerv__Stub__<xsl:value-of select="class_name"/>__<xsl:value-of select="class_name"/>_h
#define Define__Nerv__Stub__<xsl:value-of select="class_name"/>__<xsl:value-of select="class_name"/>_h

#include &lt;Nerv/Method<xsl:value-of select="$method_domain_path"/>/<xsl:value-of select="method_domain_name"/>.h&gt;

#include &lt;boost/python.hpp&gt;
namespace py = boost::python;



class <xsl:value-of select="$class_name"/> : public Nerv::Method<xsl:value-of select="$method_domain_namespace"/>::stub_interface_<xsl:value-of select="$interface_name"/>, public py::wrapper&lt;<xsl:value-of select="$class_name"/>&gt;
{
public:
    <xsl:value-of select="$class_name"/>();
    ~<xsl:value-of select="$class_name"/>();

    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">

            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>

            <xsl:if test="type!='BROADCAST'" >
    /********************************************************************************************************************************************
        Method       : <xsl:value-of select="name"/>
        Input        : <xsl:if test="in_data != ''"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/></xsl:if>
        output       : <xsl:if test="out_data != ''"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/></xsl:if>
        Method Type  : <xsl:value-of select="type"/>
        Code         : <xsl:value-of select="code"/>
        Description  : <xsl:value-of select="description"/>
    ********************************************************************************************************************************************/
            </xsl:if>

            <xsl:choose>
              <xsl:when test="type='CALL'">
                <xsl:choose>
                  <xsl:when test="in_data='(null)' and out_data='(null)'">
    Nerv::nerv_err_t on_<xsl:value-of select="$method_name"/>(const Nerv::client_addr_t&amp; client_addr, const double&amp; in_elapsed_time, const double&amp; in_limit_time);
                  </xsl:when>
                  <xsl:when test="in_data='(null)' and out_data!='(null)'">
    Nerv::nerv_err_t on_<xsl:value-of select="$method_name"/>(const Nerv::client_addr_t&amp; client_addr, <xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* p_out,const double&amp; in_elapsed_time, const double&amp; in_limit_time);
                  </xsl:when>
                  <xsl:when test="in_data!='(null)' and out_data='(null)'">
    Nerv::nerv_err_t on_<xsl:value-of select="$method_name"/>(const Nerv::client_addr_t&amp; client_addr, <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* p_in, const double&amp; in_elapsed_time, const double&amp; in_limit_time);
                  </xsl:when>
                  <xsl:when test="in_data!='(null)' and out_data!='(null)'">
    Nerv::nerv_err_t on_<xsl:value-of select="$method_name"/>(const Nerv::client_addr_t&amp; client_addr, <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* p_in, <xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* p_out,const double&amp; in_elapsed_time, const double&amp; in_limit_time);
                  </xsl:when>
                </xsl:choose>
    void on_<xsl:value-of select="$method_name"/>_cancel(const Nerv::client_addr_t&amp; in_client_addr, const Nerv::nerv_err_t&amp; in_error);
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
    void on_<xsl:value-of select="$method_name"/>(const Nerv::client_addr_t&amp; client, <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* pIn,const double&amp; in_elapsed_time);
    void on_<xsl:value-of select="$method_name"/>_cancel(const Nerv::client_addr_t&amp; in_client_addr, const Nerv::nerv_err_t&amp; in_error);
              </xsl:when>
              <xsl:when test="type='EVENT'">
    Nerv::nerv_err_t on_<xsl:value-of select="$method_name"/>_connect(const Nerv::client_addr_t&amp; in_client_addr, const double&amp; in_elapsed_time, const double&amp; in_limit_time);
    void on_<xsl:value-of select="$method_name"/>_shutdown(const Nerv::client_addr_t&amp; client_addr);
              </xsl:when>
              <xsl:when test="type='COMMAND'">
    Nerv::nerv_err_t on_<xsl:value-of select="$method_name"/>_connect(const Nerv::client_addr_t&amp; in_client_addr, const double&amp; in_elapsed_time, const double&amp; in_limit_time);
    void on_<xsl:value-of select="$method_name"/>_shutdown(const Nerv::client_addr_t&amp; in_client_addr);
                <xsl:choose >
                  <xsl:when test="in_data='(null)'">
    void on_<xsl:value-of select="$method_name"/>_sync(const Nerv::client_addr_t&amp; in_client, const Nerv::sender_sync_model_t&amp; in_sync_model, const double&amp; in_elapsed, const double&amp; in_limit);
                  </xsl:when>
                  <xsl:otherwise>
    void on_<xsl:value-of select="$method_name"/>_sync(const Nerv::client_addr_t&amp; in_client, const Nerv::sender_sync_model_t&amp; in_sync_model, const double&amp; in_elapsed, const double&amp; in_limit,<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/> const* const pin_buffer);
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
    Nerv::nerv_err_t on_<xsl:value-of select="$method_name"/>_connect(const Nerv::client_addr_t&amp; in_client_addr, const bool&amp; in_tcp_request, const double&amp; in_elapsed_time, const double&amp; in_limit_time);
    void on_<xsl:value-of select="$method_name"/>_shutdown(const Nerv::client_addr_t&amp; in_client_addr);
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
};

#endif // Define__Nerv__Stub__<xsl:value-of select="class_name"/>__<xsl:value-of select="class_name"/>_h

  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
