<?xml version="1.0" encoding="UTF-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_method_domain_path" />

<xsl:template match="/">
<xsl:for-each select="nerv_icde/methoddomain/row">
<xsl:if test="path=$export_method_domain_path">
<xsl:variable name="open_namespace_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="' { namespace '" />
</xsl:call-template>
</xsl:variable>
<xsl:variable name="close_namespace_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="' */ } /* '" />
</xsl:call-template>
</xsl:variable>
<xsl:variable name="define_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="'__'" />
</xsl:call-template>
</xsl:variable>
/****************************************************************************************************************************************************
    Document Header
    domain   : <xsl:value-of select="name"/>
    creator  : <xsl:value-of select="author"/>
    create date          : <xsl:value-of select="create-day"/>
    description          : <xsl:value-of select="description"/>
****************************************************************************************************************************************************/


#pragma once

/* Include Nerv Sdk Document ***********************************************************************************************************************/
#include &lt;Nerv/Nerv.h&gt;

/* Include Data Domains Document *****************************************************************************************************************/
<xsl:for-each select="ref_data_domain/row">
    <xsl:variable name="ref_domain_id" select="data_domain_id"/>
    <xsl:for-each select="/nerv_icde/datadomain/row">
        <xsl:if test="id=$ref_domain_id">
#include &lt;Nerv/Data<xsl:value-of select="path"/>/<xsl:value-of select="name"/>.h&gt;
        </xsl:if>
    </xsl:for-each>
</xsl:for-each>


/* Domain Define ***********************************************************************************************************************************/

namespace Nerv { namespace Method <xsl:value-of select="$open_namespace_path" /> {

    /* Standard Domain Port Constance **************************************************************************************************************/
    const ::Nerv::domain_t c_standard_port = <xsl:value-of select="standard_port"/>;


    /* Method Defines ******************************************************************************************************************************/

<xsl:for-each select="method/row">
    <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    /********************************************************************************************************************************************
        Method       : <xsl:value-of select="name"/>
        Input        : <xsl:if test="in_data != '(null)'"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/></xsl:if>
        output       : <xsl:if test="out_data != '(null)'"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/></xsl:if>
        Method Type  : <xsl:value-of select="type"/>
        Code         : <xsl:value-of select="code"/>
        Writer       : <xsl:value-of select="../../author"/>
        Description  : <xsl:value-of select="../../description"/>
    ********************************************************************************************************************************************/
  <xsl:if test="type='INFORMATION'" ><xsl:call-template name="template_information" /></xsl:if>
  <xsl:if test="type='COMMAND'" ><xsl:call-template name="template_command" /></xsl:if>
  <xsl:if test="type='CALL'" ><xsl:call-template name="template_call" /></xsl:if>
  <xsl:if test="type='SIGNAL'" ><xsl:call-template name="template_signal" /></xsl:if>
  <xsl:if test="type='EVENT'" ><xsl:call-template name="template_event" /></xsl:if>
  <xsl:if test="type='BROADCAST'" ><xsl:call-template name="template_broadcast" /></xsl:if>
</xsl:for-each>


    /* Interface Defines ***************************************************************************************************************************/

<!--         /********************************************************************************************************************************************
         Interface Name : <xsl:value-of select="interface/name"/>
         Writer : <xsl:value-of select="author"/>
         Description : <xsl:value-of select="interface/description"/>
        *******************************************************************************************************************************************/

        /* proxy */
        class proxy_interface_<xsl:value-of select="interface/name"/> : virtual public ::Nerv::nerv_proxy_object<xsl:for-each select="interface/membermethod/row">
        , virtual public proxy_method_<xsl:value-of select="method_name"/>
  </xsl:for-each>
        {
        };

        /* stub */
        class stub_interface_<xsl:value-of select="interface/name"/> : virtual public ::Nerv::nerv_object<xsl:for-each select="interface/membermethod/row">
        , virtual public method_<xsl:value-of select="method_name"/>
  </xsl:for-each>
        {
        }; -->


<xsl:for-each select="interface/row">
    /********************************************************************************************************************************************
        Interface Name : <xsl:value-of select="name"/>
        Writer : <xsl:value-of select="../../author"/>
        Description : <xsl:value-of select="description"/>
    *******************************************************************************************************************************************/

    /* proxy */
    class proxy_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv::nerv_proxy_object<xsl:for-each select="membermethod/row">
    , virtual public proxy_method_<xsl:value-of select="method_name"/>
    </xsl:for-each>
    {
    };

    /* stub */
    class stub_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv::nerv_object<xsl:for-each select="membermethod/row">
    , virtual public method_<xsl:value-of select="method_name"/>
    </xsl:for-each>
    {
    };
</xsl:for-each>

} } /* <xsl:value-of select="$close_namespace_path" /> */


</xsl:if>
</xsl:for-each>
</xsl:template>


<xsl:template name ="template_information" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_information
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_information(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>_connect( const bool&amp; in_udp_request = true, const double&amp; in_limit_time = 1.0, double* pout_time = 0)
        {
            return_object_t return_object = async_connect( in_udp_request , in_limit_time );
            return async_connect_return( return_object, pout_time );
        }

        void <xsl:value-of select="name"/>_shutdown() { shutdown(); }

        void <xsl:value-of select="name"/>_set_sync(const bool&amp; in_use_on_sync) { set_sync(in_use_on_sync); }

        letter_case_t <xsl:value-of select="name"/>_see_history( const size_t&amp; in_size)
        {
            return see_history( in_size);
        }

        void <xsl:value-of select="name"/>_set_history_size( const size_t&amp; in_history_size)
        {
            set_history_size( in_history_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_read( <xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* pin_data)
        {
            Nerv::pksize_t in_size = sizeof(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);
            return read(pin_data, &amp;in_size );
        }

    private:

        virtual void on_disconnected()
        {
            on_<xsl:value-of select="name"/>_disconnected();
        }

        virtual void on_sync( const Nerv::sender_sync_model_t&amp; in_sync_model,
                              const double&amp; in_elapsed,
                              const double&amp; in_limit,
                              void const* const pin_buffer,
                              const pksize_t in_size)
        {
            on_<xsl:value-of select="name"/>_sync(in_sync_model, in_elapsed, in_limit, (<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/> const* const)pin_buffer );
        }

        virtual void on_<xsl:value-of select="name"/>_disconnected() = 0;

        virtual void on_<xsl:value-of select="name"/>_sync( const Nerv::sender_sync_model_t&amp; in_sync_model,
                                               const double&amp; in_elapsed,
                                               const double&amp; in_limit,
                                               <xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/> const* const pin_buffer ) = 0;
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_information
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : nerv_object_information(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_set_sync_model( const Nerv::sender_sync_model_t&amp; in_sync_model, const double&amp; in_interval)
        {
            set_sync_model(in_sync_model,in_interval);
        }

        void <xsl:value-of select="name"/>_disconnect( const client_addr_t&amp; in_client_addr )
        {
            disconnect( in_client_addr );
        }

        void <xsl:value-of select="name"/>_disconnect_all_client_object()
        {
            disconnect_all_client_object();
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_node( const struct_node_addr&amp; in_node_addr)
        {
            disconnect_one_client_node( in_node_addr);
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_object( const client_addr_t&amp; in_client_addr)
        {
            disconnect_one_client_object( in_client_addr);
        }

        <xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* <xsl:value-of select="name"/>_get()
        {
            return (<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*) get(sizeof(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>) );
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            cancel();
        }

        nerv_err_t <xsl:value-of select="name"/>_post(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* pin_buf)
        {
            ::Nerv::pksize_t in_size = nerv_sizeof(pin_buf);
            return post( in_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_write(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* pin_buf)
        {
            ::Nerv::pksize_t in_size = nerv_sizeof(pin_buf);
            return write( pin_buf, in_size);
        }

        letter_case_t <xsl:value-of select="name"/>_see_history( const size_t&amp; in_size)
        {
            return see_history( in_size);
        }

        void <xsl:value-of select="name"/>_set_history_size( const size_t&amp; in_history_size)
        {
            set_history_size( in_history_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_read( <xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* pin_data)
        {
            Nerv::pksize_t in_size = sizeof(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);
            return read(pin_data, &amp;in_size );
        }

    private:

        virtual nerv_err_t on_connect( const client_addr_t&amp; in_client_addr,
                                       const bool&amp; in_tcp_request,
                                       const double&amp; in_elapsed_time,
                                       const double&amp; in_limit_time )
        {
            return on_<xsl:value-of select="name"/>_connect( in_client_addr, in_tcp_request, in_elapsed_time, in_limit_time );
        }

        virtual void on_shutdown( const client_addr_t&amp; in_client_addr)
        {
            on_<xsl:value-of select="name"/>_shutdown(in_client_addr);
        }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_connect( const client_addr_t&amp; in_client_addr,
                                                                     const bool&amp; in_tcp_request,
                                                                     const double&amp; in_elapsed_time,
                                                                     const double&amp; in_limit_time ) = 0;

        virtual void on_<xsl:value-of select="name"/>_shutdown( const client_addr_t&amp; in_client_addr) = 0;


    };
</xsl:template>

<xsl:template name ="template_command" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_command
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_command(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>_connect( const double&amp; in_limit_time = 1.0, double* pout_time = 0)
        {
            return_object_t return_object = async_connect( in_limit_time );
            return async_connect_return( return_object, pout_time );
        }

        void <xsl:value-of select="name"/>_shutdown() { shutdown(); }

        void <xsl:value-of select="name"/>_set_sync_model( const bool&amp; in_using_udp,
                                                           const Nerv::sender_sync_model_t&amp; in_sync_model,
                                                           const double&amp; in_interval )
        {
            set_sync_model( in_using_udp, in_sync_model, in_interval);
        }

        <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* <xsl:value-of select="name"/>_get()
        {
            return (<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)get(sizeof(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>));
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            cancel();
        }

        nerv_err_t <xsl:value-of select="name"/>_post(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* pin_buf  )
        {
            ::Nerv::pksize_t in_size = nerv_sizeof(pin_buf);
            return post( in_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_write( <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/> const* const&amp; pin_buf)
        {
            ::Nerv::pksize_t in_size = nerv_sizeof(pin_buf);
            return write( pin_buf, in_size);
        }

        letter_case_t <xsl:value-of select="name"/>_see_history( const size_t&amp; in_size)
        {
            return see_history( in_size);
        }

        void <xsl:value-of select="name"/>_set_history_size( const size_t&amp; in_history_size)
        {
            set_history_size( in_history_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_read( <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* pin_data)
        {
            Nerv::pksize_t in_size = sizeof(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>);
            return read(pin_data, &amp;in_size );
        }

    private:

        virtual void on_disconnected()
        {
            on_<xsl:value-of select="name"/>_disconnected();
        }

        virtual void on_<xsl:value-of select="name"/>_disconnected() = 0;
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_command
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : nerv_object_command(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_disconnect( const client_addr_t&amp; in_client_addr )
        {
            disconnect( in_client_addr );
        }

        void <xsl:value-of select="name"/>_disconnect_all_client_object()
        {
            disconnect_all_client_object();
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_node( const struct_node_addr&amp; in_node_addr)
        {
            disconnect_one_client_node( in_node_addr);
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_object( const client_addr_t&amp; in_client_addr)
        {
            disconnect_one_client_object( in_client_addr);
        }

        void <xsl:value-of select="name"/>_set_sync(const bool&amp; in_use_on_sync) { set_sync(in_use_on_sync); }

        letter_case_t <xsl:value-of select="name"/>_see_history( const client_addr_t&amp; in_client_addr, const size_t&amp; in_size)
        {
            return see_history(in_client_addr, in_size);
        }

        void <xsl:value-of select="name"/>_set_history_size( const size_t&amp; in_history_size)
        {
            set_history_size( in_history_size);
        }

        int <xsl:value-of select="name"/>_get_client_count()
        {
            return get_client_count();
        }

        nerv_err_t <xsl:value-of select="name"/>_read(const client_addr_t&amp; in_client, <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* pin_data)
        {
            Nerv::pksize_t in_size = sizeof(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>);
            return read(in_client, pin_data, &amp;in_size );
        }

    private:

        virtual void on_sync( const client_addr_t&amp; in_client,
                              const Nerv::sender_sync_model_t&amp; in_sync_model,
                              const double&amp; in_elapsed,
                              const double&amp; in_limit,
                              void const* const pin_buffer,
                              const pksize_t in_size)
        {
            on_<xsl:value-of select="name"/>_sync(in_client, in_sync_model,in_limit, in_elapsed,(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/> const* const)pin_buffer);
        }

        virtual nerv_err_t on_connect( const client_addr_t&amp; in_client_addr,
                                       const double&amp; in_elapsed_time,
                                       const double&amp; in_limit_time )
        {
            return on_<xsl:value-of select="name"/>_connect( in_client_addr, in_elapsed_time, in_limit_time );
        }

        virtual void on_shutdown( const client_addr_t&amp; in_client_addr)
        {
            on_<xsl:value-of select="name"/>_shutdown(in_client_addr);
        }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_connect( const client_addr_t&amp; in_client_addr,
                                                                     const double&amp; in_elapsed_time,
                                                                     const double&amp; in_limit_time ) = 0;

        virtual void on_<xsl:value-of select="name"/>_shutdown( const client_addr_t&amp; in_client_addr) = 0;

        virtual void on_<xsl:value-of select="name"/>_sync( const client_addr_t&amp; in_client,
                                                   const Nerv::sender_sync_model_t&amp; in_sync_model,
                                                   const double&amp; in_elapsed,
                                                   const double&amp; in_limit,
                                                   <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/> const* const pin_buffer ) = 0; //{}
    };
</xsl:template>

<xsl:template name ="template_call" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_call
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_call(<xsl:value-of select="code"/>) {}

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>(<xsl:if test="in_data!='(null)'"> <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* p_in,</xsl:if><xsl:if test="out_data!='(null)'"> <xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* p_out,</xsl:if> const double&amp; in_limit_time = 1.0, double* pout_time = 0)
        {
            <xsl:if test="in_data!='(null)'">::Nerv::pksize_t in_size = nerv_sizeof(p_in);</xsl:if>
            <xsl:if test="out_data!='(null)'">::Nerv::pksize_t out_size = nerv_sizeof(p_out);</xsl:if>
            return_object_t return_object = async_call(<xsl:choose><xsl:when test="in_data!='(null)'">p_in,in_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> in_limit_time);
            return async_return( return_object, <xsl:choose><xsl:when test="out_data!='(null)'">p_out,out_size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> pout_time );
        }

        void <xsl:value-of select="name"/>_cancel(const nerv_err_t&amp; in_error)
        {
            cancel(in_error);
        }
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_call
    {
    public:

        method_<xsl:value-of select="name"/>() : nerv_object_call(<xsl:value-of select="code"/>, <xsl:if test="out_data!='(null)'">sizeof(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>)</xsl:if><xsl:if test="out_data='(null)'">0</xsl:if> ) {}

    protected:
        virtual ::Nerv::nerv_err_t on_call ( const client_addr_t&amp;  in_client_addr,
                                 void* const   pin_buf,
                                 const pksize_t&amp;  in_buf_size,
                                 void*  pout_buf,
                                 pksize_t*  pout_buf_size,
                                 const double&amp;  in_elapsed_time,
                                 const double&amp;  in_limit_time )
        {
            ::Nerv::nerv_err_t r = on_<xsl:value-of select="name"/>(in_client_addr,<xsl:if test="in_data!='(null)'"> (<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)pin_buf,</xsl:if><xsl:if test="out_data!='(null)'"> (<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)pout_buf,</xsl:if> in_elapsed_time, in_limit_time);
  <xsl:if test="out_data!='(null)'">
            if( ! r )
            {
                *pout_buf_size = nerv_sizeof((<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)pout_buf);
            }
            return r;
  </xsl:if>
        }

        virtual void on_call_cancel( const client_addr_t&amp; in_client_addr, const nerv_err_t&amp; in_error)
        {
            on_<xsl:value-of select="name"/>_cancel(in_client_addr, in_error);
        }

        virtual ::Nerv::nerv_err_t on_<xsl:value-of select="name"/>(const ::Nerv::client_addr_t&amp; client_addr, <xsl:if test="in_data!='(null)'"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* p_in,</xsl:if> <xsl:if test="out_data!='(null)'"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* p_out,</xsl:if> const double&amp; in_elapsed_time, const double&amp; in_limit_time) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel(const ::Nerv::client_addr_t&amp; in_client_addr, const ::Nerv::nerv_err_t&amp; in_error) = 0;
    };
</xsl:template>

<xsl:template name ="template_signal" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_signal
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_signal(<xsl:value-of select="code"/>) {}

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>(<xsl:if test="in_data != ''"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* p_in,</xsl:if>  const double&amp; in_limit_time = 1.0, double* pout_time= 0)
        {
            <xsl:if test="in_data != ''">::Nerv::pksize_t inSize = nerv_sizeof(p_in);</xsl:if>
            return signal(<xsl:choose><xsl:when test="in_data != ''">p_in, inSize,</xsl:when><xsl:when test="in_data = ''">0, 0,</xsl:when></xsl:choose> in_limit_time, pout_time);
        }

        void <xsl:value-of select="name"/>_cancel(const nerv_err_t&amp; in_error)
        {
            cancel(in_error);
        }
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_signal
    {
    public:
        method_<xsl:value-of select="name"/>() : nerv_object_signal(<xsl:value-of select="code"/>) {}

    protected:
        virtual void on_signal( const ::Nerv::client_addr_t&amp;   in_client_addr,
                    void* const   pin_buf,
                    const ::Nerv::pksize_t&amp;   in_buf_size,
                    const double&amp;   in_elapsed_time )
        {
            on_<xsl:value-of select="name"/>(in_client_addr, <xsl:if test="in_data != ''">(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)pin_buf,</xsl:if> in_elapsed_time);
        }

        virtual void on_signal_cancel( const ::Nerv::client_addr_t&amp; in_client_addr, const ::Nerv::nerv_err_t&amp; in_error)
        {
            on_<xsl:value-of select="name"/>_cancel(in_client_addr,in_error);
        }

        virtual void on_<xsl:value-of select="name"/>(const ::Nerv::client_addr_t&amp; client, <xsl:if test="in_data != ''"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* p_in,</xsl:if> const double&amp; in_elapsed_time) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel(const ::Nerv::client_addr_t&amp; in_client_addr, const ::Nerv::nerv_err_t&amp; in_error) = 0;
    };
</xsl:template>

<xsl:template name ="template_event" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_event
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_event(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv::nerv_err_t <xsl:value-of select="name"/>_connect( const double&amp; in_limit_time = 1.0, double* pout_time= 0 )
        {
            return_object_t return_object = async_connect( in_limit_time );
            return async_connect_return( return_object, pout_time );
        }

        void <xsl:value-of select="name"/>_shutdown() { shutdown(); }

        virtual void on_event( void const * const pin_event_data, const pksize_t&amp; in_event_size, const double&amp;  in_elapsed_time)
        {
            on_<xsl:value-of select="name"/>(<xsl:if test="out_data!='(null)'">(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)pin_event_data,</xsl:if> in_elapsed_time);
        }

    private:
        virtual void on_disconnected() { on_<xsl:value-of select="name"/>_disconnected(); }

        virtual void on_<xsl:value-of select="name"/>( <xsl:if test="out_data!='(null)'"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* p_out,</xsl:if> const double&amp; in_elapsed_time) = 0; //{}

        virtual void on_<xsl:value-of select="name"/>_disconnected() = 0; //{}
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_event
    {
    public:
        method_<xsl:value-of select="name"/>() : nerv_object_event(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_disconnect(const ::Nerv::client_addr_t client_addr ) { disconnect(client_addr); }

        void <xsl:value-of select="name"/>_disconnect_all_client_object()
        {
            disconnect_all_client_object();
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_node( const struct_node_addr&amp; in_node_addr)
        {
            disconnect_one_client_node( in_node_addr);
        }

        void <xsl:value-of select="name"/>_disconnect_one_client_object( const client_addr_t&amp; in_client_addr)
        {
            disconnect_one_client_object( in_client_addr);
        }
  <xsl:if test="out_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* pout_buf)
        {
            ::Nerv::pksize_t outSize = nerv_sizeof(pout_buf);
            return event(pout_buf,outSize);
        }
  </xsl:if>
  <xsl:if test="out_data='(null)'">
        nerv_err_t <xsl:value-of select="name"/>()
        {
            return event(0,0);
        }
  </xsl:if>

        virtual ::Nerv::nerv_err_t on_connect(  const client_addr_t&amp; in_client_addr,
                                                    const double&amp; in_elapsed_time,
                                                    const double&amp; in_limit_time )
        {
            return on_<xsl:value-of select="name"/>_connect(in_client_addr, in_elapsed_time, in_limit_time);
        }

        virtual void on_shutdown( const ::Nerv::client_addr_t&amp; client_addr) { on_<xsl:value-of select="name"/>_shutdown( client_addr); }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_connect( const client_addr_t&amp; in_client_addr,
                                                      const double&amp; in_elapsed_time,
                                                      const double&amp; in_limit_time ) = 0;

        virtual void on_<xsl:value-of select="name"/>_shutdown(const ::Nerv::client_addr_t&amp; client_addr) = 0;
    };
</xsl:template>

<xsl:template name ="template_broadcast" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv::nerv_proxy_object_broadcast
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : nerv_proxy_object_broadcast(<xsl:value-of select="code"/>)
        {
        }

        virtual void on_<xsl:value-of select="name"/>_broadcast( const ::Nerv::server_addr_t&amp; in_server_addr,<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* const pin_buf, const double&amp; in_elapsed_time) = 0;

        virtual void on_broadcast( const struct_server_addr&amp; in_server_addr, void* const pin_buf, const pksize_t&amp; in_size, const double&amp;  in_elapsed_time)
        {
            on_<xsl:value-of select="name"/>_broadcast(in_server_addr,(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)pin_buf, in_elapsed_time);
        }
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv::nerv_object_broadcast
    {
    public:
        method_<xsl:value-of select="name"/>() : nerv_object_broadcast(<xsl:value-of select="code"/>)
        {
        }

        nerv_err_t <xsl:value-of select="name"/>_broadcast(Nerv::Data::<xsl:value-of select="in_data_domain"/>::<xsl:value-of select="in_data"/>* pin_buf)
        {
            ::Nerv::pksize_t in_size = nerv_sizeof(pin_buf);
            return broadcast(pin_buf,in_size);
        }
    };
</xsl:template>




</xsl:stylesheet>
